@@ -10,7 +10,7 @@ import torch
 from pyknos.nflows.nn import nets
 from torch import Tensor, nn, optim, relu
 from torch.nn import MSELoss
-from torch.nn.utils import clip_grad_norm_
+from torch.nn.utils.clip_grad import clip_grad_norm_
 from torch.utils import data
 from torch.utils.data.sampler import SubsetRandomSampler
 
@@ -27,7 +27,7 @@ class Destandardize(nn.Module):
         self.register_buffer("_std", std)
 
     def forward(self, tensor):
-        return tensor * self.std + self.mean
+        return tensor * self._std + self._mean
 
 
 def destandardizing_net(batch_t: Tensor, min_std: float = 1e-7) -> nn.Module:
@@ -52,9 +52,9 @@ def destandardizing_net(batch_t: Tensor, min_std: float = 1e-7) -> nn.Module:
     else:
         t_std = 1
         logging.warning(
-            f"""Using a one-dimensional batch will instantiate a Standardize transform 
-            with (mean, std) parameters which are not representative of the data. We 
-            allow this behavior because you might be loading a pre-trained. If this is 
+            """Using a one-dimensional batch will instantiate a Standardize transform
+            with (mean, std) parameters which are not representative of the data. We
+            allow this behavior because you might be loading a pre-trained. If this is
             not the case, please be sure to use a larger batch."""
         )
 
@@ -62,8 +62,8 @@ def destandardizing_net(batch_t: Tensor, min_std: float = 1e-7) -> nn.Module:
 
 
 def build_input_output_layer(
-    batch_theta: Tensor = None,
-    batch_property: Tensor = None,
+    batch_theta: Tensor,
+    batch_property: Tensor,
     z_score_theta: bool = True,
     z_score_property: bool = True,
     embedding_net_theta: nn.Module = nn.Identity(),
@@ -100,8 +100,7 @@ def build_input_output_layer(
 
 class ActiveSubspace:
     def __init__(self, posterior: Any):
-        """
-        Identify the active subspace for sensitivity analyses.
+        """Identify the active subspace for sensitivity analyses.
 
         - Introduction to active subspaces: Constantine et al. 2015.
         - Application to analyse the sensitivity in neuroscience models:
@@ -111,12 +110,13 @@ class ActiveSubspace:
             posterior: Posterior distribution obtained with `SNPE`, `SNLE`, or `SNRE`.
                 Needs to have a `.sample()` method. If we want to analyse the
                 sensitivity of the posterior probability, it also must have a
-                `.log_prob()` method.
+                `.potential()` method.
         """
         self._posterior = posterior
         self._regression_net = None
         self._theta = None
         self._emergent_property = None
+        self._device = posterior._device
         self._validation_log_probs = []
 
     def add_property(
@@ -131,8 +131,7 @@ class ActiveSubspace:
         z_score_property: bool = True,
         embedding_net: nn.Module = nn.Identity(),
     ) -> "ActiveSubspace":
-        r"""
-        Add a property whose sensitivity is to be analysed.
+        r"""Add a property whose sensitivity is to be analysed.
 
         To analyse the sensitivity of an emergent property, we train a neural network
         to predict the property from the parameter set $\theta$. The hyperparameters of
@@ -173,58 +172,56 @@ class ActiveSubspace:
         self._theta = theta
         self._emergent_property = emergent_property
 
+        def build_resnet(theta):
+            classifier = nets.ResidualNet(
+                in_features=theta.shape[1],
+                out_features=1,
+                hidden_features=hidden_features,
+                context_features=None,
+                num_blocks=num_blocks,
+                activation=relu,
+                dropout_probability=dropout_probability,
+                use_batch_norm=True,
+            )
+            input_layer, output_layer = build_input_output_layer(
+                theta,
+                emergent_property,
+                z_score_theta,
+                z_score_property,
+                embedding_net,
+            )
+            classifier = nn.Sequential(input_layer, classifier, output_layer)
+            return classifier
+
+        def build_mlp(theta):
+            classifier = nn.Sequential(
+                nn.Linear(theta.shape[1], hidden_features),
+                nn.BatchNorm1d(hidden_features),
+                nn.ReLU(),
+                nn.Linear(hidden_features, hidden_features),
+                nn.BatchNorm1d(hidden_features),
+                nn.ReLU(),
+                nn.Linear(hidden_features, 1),
+            )
+            input_layer, output_layer = build_input_output_layer(
+                theta,
+                emergent_property,
+                z_score_theta,
+                z_score_property,
+                embedding_net,
+            )
+            classifier = nn.Sequential(input_layer, classifier, output_layer)
+            return classifier
+
         if isinstance(model, str):
             if model == "resnet":
-
-                def build_nn(theta):
-                    classifier = nets.ResidualNet(
-                        in_features=theta.shape[1],
-                        out_features=1,
-                        hidden_features=hidden_features,
-                        context_features=None,
-                        num_blocks=num_blocks,
-                        activation=relu,
-                        dropout_probability=dropout_probability,
-                        use_batch_norm=True,
-                    )
-                    input_layer, output_layer = build_input_output_layer(
-                        theta,
-                        emergent_property,
-                        z_score_theta,
-                        z_score_property,
-                        embedding_net,
-                    )
-                    classifier = nn.Sequential(input_layer, classifier, output_layer)
-                    return classifier
-
+                self._build_nn = build_resnet
             elif model == "mlp":
-
-                def build_nn(theta):
-                    classifier = nn.Sequential(
-                        nn.Linear(theta.shape[1], hidden_features),
-                        nn.BatchNorm1d(hidden_features),
-                        nn.ReLU(),
-                        nn.Linear(hidden_features, hidden_features),
-                        nn.BatchNorm1d(hidden_features),
-                        nn.ReLU(),
-                        nn.Linear(hidden_features, 1),
-                    )
-                    input_layer, output_layer = build_input_output_layer(
-                        theta,
-                        emergent_property,
-                        z_score_theta,
-                        z_score_property,
-                        embedding_net,
-                    )
-                    classifier = nn.Sequential(input_layer, classifier, output_layer)
-                    return classifier
-
+                self._build_nn = build_mlp
             else:
                 raise NameError
         else:
-            build_nn = model
-
-        self._build_nn = build_nn
+            self._build_nn = model
 
         return self
 
@@ -234,11 +231,10 @@ class ActiveSubspace:
         learning_rate: float = 5e-4,
         validation_fraction: float = 0.1,
         stop_after_epochs: int = 20,
-        max_num_epochs: Optional[int] = None,
+        max_num_epochs: int = 2**31 - 1,
         clip_max_norm: Optional[float] = 5.0,
     ) -> nn.Module:
-        r"""
-        Train a regression network to predict the specified property from $\theta$.
+        r"""Train a regression network to predict the specified property from $\theta$.
 
         Args:
             training_batch_size: Training batch size.
@@ -247,12 +243,16 @@ class ActiveSubspace:
             stop_after_epochs: The number of epochs to wait for improvement on the
                 validation set before terminating training.
             max_num_epochs: Maximum number of epochs to run. If reached, we stop
-                training even when the validation loss is still decreasing. If None, we
-                train until validation loss increases (see also `stop_after_epochs`).
+                training even when the validation loss is still decreasing. Otherwise,
+                we train until validation loss increases (see also `stop_after_epochs`).
             clip_max_norm: Value at which to clip the total gradient norm in order to
                 prevent exploding gradients. Use `None` for no clipping.
         """
 
+        assert (
+            self._theta is not None and self._emergent_property is not None
+        ), "You must call .add_property() first."
+
         # Get indices for permutation of the data.
         num_examples = len(self._theta)
         permuted_indices = torch.randperm(num_examples)
@@ -271,24 +271,26 @@ class ActiveSubspace:
             dataset,
             batch_size=training_batch_size,
             drop_last=True,
-            sampler=SubsetRandomSampler(train_indices),
+            sampler=SubsetRandomSampler(train_indices.tolist()),
         )
         val_loader = data.DataLoader(
             dataset,
             batch_size=min(training_batch_size, num_examples - num_training_examples),
             shuffle=False,
             drop_last=True,
-            sampler=SubsetRandomSampler(val_indices),
+            sampler=SubsetRandomSampler(val_indices.tolist()),
         )
 
         if self._regression_net is None:
-            self._regression_net = self._build_nn(self._theta[train_indices])
+            self._regression_net = self._build_nn(self._theta[train_indices]).to(
+                self._device
+            )
 
         optimizer = optim.Adam(
             list(self._regression_net.parameters()),
             lr=learning_rate,
         )
-        max_num_epochs = 2 ** 31 - 1 if max_num_epochs is None else max_num_epochs
+        max_num_epochs = 2**31 - 1 if max_num_epochs is None else max_num_epochs
 
         # criterion / loss
         criterion = MSELoss()
@@ -298,8 +300,8 @@ class ActiveSubspace:
             self._regression_net.train()
             for parameters, observations in train_loader:
                 optimizer.zero_grad()
-                outputs = self._regression_net(parameters)
-                loss = criterion(outputs, observations)
+                outputs = self._regression_net(parameters.to(self._device))
+                loss = criterion(outputs, observations.to(self._device))
                 loss.backward()
                 if clip_max_norm is not None:
                     clip_grad_norm_(
@@ -316,20 +318,21 @@ class ActiveSubspace:
             val_loss = 0.0
             with torch.no_grad():
                 for parameters, observations in val_loader:
-                    outputs = self._regression_net(parameters)
-                    loss = criterion(outputs, observations)
+                    outputs = self._regression_net(parameters.to(self._device))
+                    loss = criterion(outputs, observations.to(self._device))
                     val_loss += loss.item()
             self._val_log_prob = -val_loss / num_validation_examples
             self._validation_log_probs.append(self._val_log_prob)
 
-            print("Training neural network. Epochs trained: ", epoch, end="\r")
+            print("\r", "Training neural network. Epochs trained: ", epoch, end="")
 
         return deepcopy(self._regression_net)
 
     def _converged(self, epoch: int, stop_after_epochs: int) -> bool:
-        r"""
-        Return whether the training converged yet and save best model state so far.
+        r"""Return whether the training converged yet and save best model state so far.
+
         Checks for improvement in validation performance over previous epochs.
+
         Args:
             epoch: Current epoch in training.
             stop_after_epochs: How many fruitless epochs to let pass before stopping.
@@ -338,6 +341,7 @@ class ActiveSubspace:
         """
         converged = False
 
+        assert self._regression_net is not None
         posterior_nn = self._regression_net
 
         # (Re)-start the epoch count with the first epoch or any improvement.
@@ -361,14 +365,14 @@ class ActiveSubspace:
         norm_gradients_to_prior: bool = True,
         num_monte_carlo_samples: int = 1000,
     ) -> Tuple[Tensor, Tensor]:
-        r"""
-        Return eigenvectors and values corresponding to directions of sensitivity.
+        r"""Return eigenvectors and values corresponding to directions of sensitivity.
 
         The directions of sensitivity are the directions along which a specific
         property changes in the fastest way. They will have the largest eigenvalues.
 
         This computes the matrix:
-        $\mathbf{M} = \mathbb{E}_{p(\theta|x_o)}[\nabla_{\theta} f(\theta)^T \nabla_{\theta}
+        $\mathbf{M} = \mathbb{E}_{p(\theta|x_o)}[\nabla_{\theta} f(\theta)^T
+        \nabla_{\theta}
         f(\theta)]$
         where $f(\cdot)$ is the trained regression network. The expected value is
         approximated with a Monte-Carlo mean. Next, do an eigenvalue
@@ -425,20 +429,23 @@ class ActiveSubspace:
         thetas.requires_grad = True
 
         if posterior_log_prob_as_property:
-            predictions = self._posterior.log_prob(thetas, track_gradients=True)
+            predictions = self._posterior.potential(thetas, track_gradients=True)
         else:
+            assert (
+                self._regression_net is not None
+            ), "self._regression_net is None, you must call `.train()` first."
             predictions = self._regression_net.forward(thetas)
         loss = predictions.mean()
         loss.backward()
         gradients = torch.squeeze(thetas.grad)
         if norm_gradients_to_prior:
-            if hasattr(self._posterior._prior, "stddev") and hasattr(
-                self._posterior._prior, "mean"
+            if hasattr(self._posterior.prior, "stddev") and hasattr(
+                self._posterior.prior, "mean"
             ):
-                self._prior_mean = self._posterior._prior.mean
-                self._prior_scale = self._posterior._prior.stddev
+                self._prior_mean = self._posterior.prior.mean
+                self._prior_scale = self._posterior.prior.stddev
             else:
-                prior_samples = self._posterior._prior.sample((10000,))
+                prior_samples = self._posterior.prior.sample((10000,))
                 self._prior_scale = torch.std(prior_samples, dim=0)
                 self._prior_mean = torch.mean(prior_samples, dim=0)
             gradients *= self._prior_scale
@@ -469,8 +476,7 @@ class ActiveSubspace:
         return eigen_values, eigen_vectors
 
     def project(self, theta: Tensor, num_dimensions: int) -> Tensor:
-        r"""
-        Return $\theta$ that were projected into the subspace.
+        r"""Return $\theta$ that were projected into the subspace.
 
         To identify the dimensionality of the active subspace `num_dimensions`,
         Constantine et al. 2015 suggest to look at gaps in the eigenvalue spectrum.
@@ -486,6 +492,7 @@ class ActiveSubspace:
         Returns:
             Projected parameters of shape `(theta.shape[0], num_dimensions)`.
         """
+        theta = theta.to(self._device)
         if self._gradients_are_normed:
             theta = (theta - self._prior_mean) / self._prior_scale
 
