@@ -11,7 +11,7 @@ from sbi.inference.posteriors.base_posterior import NeuralPosterior
 from sbi.inference.potentials.posterior_based_potential import (
     posterior_estimator_based_potential,
 )
-from sbi.samplers.rejection.rejection import rejection_sample_posterior_within_prior
+from sbi.samplers.rejection.rejection import accept_reject_sample
 from sbi.types import Shape, TorchTransform
 from sbi.utils import check_prior, match_theta_and_x_batch_shapes, within_support
 from sbi.utils.torchutils import ensure_theta_batched
@@ -35,34 +35,33 @@ class DirectPosterior(NeuralPosterior):
         self,
         posterior_estimator: flows.Flow,
         prior: Distribution,
-        theta_transform: Optional[TorchTransform] = None,
         max_sampling_batch_size: int = 10_000,
         device: Optional[str] = None,
         x_shape: Optional[torch.Size] = None,
+        enable_transform: bool = True,
     ):
         """
         Args:
             prior: Prior distribution with `.log_prob()` and `.sample()`.
             posterior_estimator: The trained neural posterior.
-            theta_transform: Custom transform to perform MAP optimization in
-                unconstrained space. If None (default), a suitable transform is
-                built from the prior support. In order to not use a transform at all,
-                pass an identity transform, e.g., `theta_transform=torch.distrbutions.
-                transforms`.
-                identity_transform()`.
             max_sampling_batch_size: Batchsize of samples being drawn from
                 the proposal at every iteration.
             device: Training device, e.g., "cpu", "cuda" or "cuda:0". If None,
                 `potential_fn.device` is used.
             x_shape: Shape of a single simulator output. If passed, it is used to check
                 the shape of the observed data and give a descriptive error.
+            enable_transform: Whether to transform parameters to unconstrained space
+                during MAP optimization.
         """
         # Because `DirectPosterior` does not take the `potential_fn` as input, it
         # builds it itself. The `potential_fn` and `theta_transform` are used only for
         # obtaining the MAP.
         check_prior(prior)
         potential_fn, theta_transform = posterior_estimator_based_potential(
-            posterior_estimator, prior, x_o=None, theta_transform=theta_transform
+            posterior_estimator,
+            prior,
+            x_o=None,
+            enable_transform=enable_transform,
         )
 
         super().__init__(
@@ -115,13 +114,14 @@ class DirectPosterior(NeuralPosterior):
                 f"`.build_posterior(sample_with={sample_with}).`"
             )
 
-        samples = rejection_sample_posterior_within_prior(
-            posterior_nn=self.posterior_estimator,
-            prior=self.prior,
-            x=x,
+        samples = accept_reject_sample(
+            proposal=self.posterior_estimator,
+            accept_reject_fn=lambda theta: within_support(self.prior, theta),
             num_samples=num_samples,
             show_progress_bars=show_progress_bars,
             max_sampling_batch_size=max_sampling_batch_size,
+            proposal_sampling_kwargs={"context": x},
+            alternative_method="build_posterior(..., sample_with='mcmc')",
         )[0]
         return samples
 
@@ -221,14 +221,14 @@ class DirectPosterior(NeuralPosterior):
 
         def acceptance_at(x: Tensor) -> Tensor:
 
-            return rejection_sample_posterior_within_prior(
-                posterior_nn=self.posterior_estimator,
-                prior=self.prior,
-                x=x.to(self._device),
+            return accept_reject_sample(
+                proposal=self.posterior_estimator,
+                accept_reject_fn=lambda theta: within_support(self.prior, theta),
                 num_samples=num_rejection_samples,
                 show_progress_bars=show_progress_bars,
                 sample_for_correction_factor=True,
                 max_sampling_batch_size=rejection_sampling_batch_size,
+                proposal_sampling_kwargs={"context": x},
             )[1]
 
         # Check if the provided x matches the default x (short-circuit on identity).
@@ -291,8 +291,8 @@ class DirectPosterior(NeuralPosterior):
                 `map`-attribute, and printed every `save_best_every`-th iteration.
                 Computing the best log-probability creates a significant overhead
                 (thus, the default is `10`.)
-            show_progress_bars: Whether or not to show a progressbar for sampling from
-                the posterior.
+            show_progress_bars: Whether to show a progressbar during sampling from the
+                posterior.
             force_update: Whether to re-calculate the MAP when x is unchanged and
                 have a cached value.
             log_prob_kwargs: Will be empty for SNLE and SNRE. Will contain
