@@ -4,7 +4,6 @@
 from typing import Callable, Optional, Tuple
 
 import torch
-import torch.distributions.transforms as torch_tf
 from pyknos.nflows import flows
 from torch import Tensor, nn
 from torch.distributions import Distribution
@@ -20,7 +19,7 @@ def posterior_estimator_based_potential(
     posterior_estimator: nn.Module,
     prior: Distribution,
     x_o: Optional[Tensor],
-    theta_transform: Optional[TorchTransform] = None,
+    enable_transform: bool = True,
 ) -> Tuple[Callable, TorchTransform]:
     r"""Returns the potential for posterior-based methods.
 
@@ -34,11 +33,7 @@ def posterior_estimator_based_potential(
         posterior_estimator: The neural network modelling the posterior.
         prior: The prior distribution.
         x_o: The observed data at which to evaluate the posterior.
-        theta_transform: Transform to map the parameters to an
-            unconstrained space. If None (default), a suitable transform is
-            built from the prior support. In order to not use a transform at all,
-            pass an identity transform, e.g., `theta_transform=torch.distrbutions.
-            transforms`.
+        enable_transform: Whether to transform parameters to unconstrained space.
 
     Returns:
         The potential function and a transformation that maps
@@ -51,8 +46,9 @@ def posterior_estimator_based_potential(
         posterior_estimator, prior, x_o, device=device
     )
 
-    if theta_transform is None:
-        theta_transform = mcmc_transform(prior, device=device)
+    theta_transform = mcmc_transform(
+        prior, device=device, enable_transform=enable_transform
+    )
 
     return potential_fn, theta_transform
 
