@@ -1,16 +1,16 @@
-import numpy
 import pytest
 import torch
 
-# Seed for `set_seed` fixture. Change to change random state of all seeded tests.
+from sbi.utils.sbiutils import seed_all_backends
+
+# Seed for `set_seed` fixture. Change to random state of all seeded tests.
 seed = 1
 
 
 # Use seed automatically for every test function.
 @pytest.fixture(autouse=True)
 def set_seed():
-    torch.manual_seed(seed)
-    numpy.random.seed(seed)
+    seed_all_backends(seed)
 
 
 @pytest.fixture(scope="session", autouse=True)
